from dataclasses import dataclass
from typing import Dict


@dataclass
class SprintStoryPoints:
    """
    A sprint story points representation class.
    """
    completed: int = 0
    not_completed: int = 0

    @property
    def total(self) -> int:
        """
        Total story points assigned.

        :return: Total story points.
        """
        return self.completed + self.not_completed

    @property
    def completed_perc(self) -> float:
        try:
            return (self.completed / self.total) * 100
        except ZeroDivisionError:
            return 0.0

    @property
    def not_completed_perc(self) -> float:
        try:
            return (self.not_completed / self.total) * 100
        except ZeroDivisionError:
            return 0.0

    def add_completed(self, points: int) -> None:
        """
        Adds completed story points.

        :param points: Completed story points.

        :return: No return.
        """
        self.completed += points

    def add_not_completed(self, points: int) -> None:
        """
        Adds not completed story points.

        :param points: Not completed story points.

        :return: No return.
        """
        self.not_completed += points

    def to_dict(self) -> Dict[str, int]:
        """
        Serializes the class.

        :return: Dictionary class representation.
        """
        return {
            'completed': self.completed,
            'not_completed': self.not_completed,
            'total': self.total
        }

    def __str__(self) -> str:
        """
        String representation.

        :return: String representation.
        """
        return (
            f'Total: {self.total:3d} | '
            f'Completed: {self.completed:3d}({self.completed_perc:5.1f}%) | '
            f'Not completed: {self.not_completed:3d}({self.not_completed_perc:5.1f}%).'
        )



from dataclasses import dataclass
from typing import Dict, Any


@dataclass
class SprintIssue:
    """
    A sprint issue representation class.
    """
    id: str
    key: str
    done: str
    assigned_account_id: str
    assignee_name: str
    story_points: int

    @property
    def assignee(self) -> str:
        """
        Assignee field.

        :return: Asignee representation field string.
        """
        return f'{self.assigned_account_id}#{self.assignee_name}'

    @staticmethod
    def from_dict(serialized_issue: Dict[str, Any]) -> 'SprintIssue':
        """
        Creates an instance from a dictionary.

        :param serialized_issue: Dictionary representation of a sprint issue.

        :return: Sprint issue instance.
        """
        estimate = serialized_issue.get('estimateStatistic', serialized_issue.get('currentEstimateStatistic', {}))
        story_points = int(estimate.get('statFieldValue', {}).get('value', 0))

        return SprintIssue(
            serialized_issue['id'],
            serialized_issue['key'],
            serialized_issue['done'],
            serialized_issue['assigneeAccountId'],
            serialized_issue['assigneeName'],
            story_points
        )



from typing import List

from b_jira_filter.objects.sprint_issue import SprintIssue
from b_jira_filter.objects.sprint_story_points import SprintStoryPoints


class Assignee:
    """
    A class representing a developer in a sprint.
    """
    def __init__(self):
        """
        Constructor.
        """
        self.__completed_issues: List[SprintIssue] = []
        self.__not_completed_issues: List[SprintIssue] = []

    def __str__(self):
        """
        String representation.

        :return: String representation.
        """
        return str(self.sprint_story_points())

    def add_completed_issue(self, issue: SprintIssue) -> None:
        """
        Adds an issue to a completed issues list.

        :param issue: A completed issue.

        :return: No return.
        """
        self.__completed_issues.append(issue)
        assert len(set([issue.assignee for issue in self.__completed_issues])) < 2, 'Multiple assignees detected.'

    def add_not_completed_issue(self, issue: SprintIssue) -> None:
        """
        Adds an issue to a not completed issues list.

        :param issue: A not completed issue.

        :return: No return.
        """
        self.__not_completed_issues.append(issue)
        assert len(set([issue.assignee for issue in self.__completed_issues])) < 2, 'Multiple assignees detected.'

    def sprint_story_points(self) -> SprintStoryPoints:
        """
        Calculates story points according to given issues.

        :return: Sprint story points.
        """
        points = SprintStoryPoints()

        for ci in self.__completed_issues:
            points.add_completed(ci.story_points)

        for ci in self.__not_completed_issues:
            points.add_not_completed(ci.story_points)

        return points



import json
from typing import Tuple, List

import requests

from b_jira_filter.objects.credentials import Credentials
from b_jira_filter.objects.sprint_issue import SprintIssue


class SprintReport:
    """
    Class that works with JIRA sprint reports.
    """
    def __init__(self, credentials: Credentials, board_id: str, sprint_id: str):
        self.credentials = credentials
        self.board_id = board_id
        self.sprint_id = sprint_id

        self.url = (
            f'{self.credentials.server}/rest/greenhopper/latest/rapid/charts'
            f'/sprintreport?rapidViewId={self.board_id}&sprintId={self.sprint_id}'
        )

    def load(self) -> Tuple[List[SprintIssue], List[SprintIssue]]:
        """
        Downloads sprint report.

        :return: Completed and not completed issues in a given sprint.
        """
        response = requests.get(
            url=self.url,
            auth=self.credentials.basic_auth
        )

        if response.status_code != 200:
            raise requests.HTTPError(response.content.decode())

        text = json.loads(response.content.decode())

        completed_issues = text['contents']['completedIssues']
        completed_issues = [SprintIssue.from_dict(ci) for ci in completed_issues]

        not_completed_issues = text['contents']['issuesNotCompletedInCurrentSprint']
        not_completed_issues = [SprintIssue.from_dict(nci) for nci in not_completed_issues]

        return completed_issues, not_completed_issues



import os
from ..common_main import common_main


def main() -> None:
    """
    Command that lists all available commands in "jira" section.

    :return: No return.
    """
    path = os.path.dirname(os.path.abspath(__file__))
    common_main(path)



from typing import List

from jira import JIRA
from jira.resources import Board


class Boards:
    """
    Class that works with JIRA boards.
    """
    def __init__(self, jira: JIRA):
        """
        Constructor.

        :param jira: JIRA SDK instance.
        """
        self.__jira = jira

    def get_boards(self) -> List[Board]:
        """
        Gets all available boards in JIRA.

        :return: List of boards.
        """
        return list(self.__jira.boards())

    def format(self, boards: List[Board]) -> str:
        """
        Formats list of boards into a human readable text.

        :param boards: List of boards.

        :return: Human readable list.
        """
        return '\n'.join([f'{index + 1}. {board}' for index, board in enumerate(boards)])

    def prompt_select_board(self) -> Board:
        """
        Asks user for an input - asks to select a board from available boards.

        :return: Selected board.
        """
        b = self.get_boards()
        b_str = self.format(b)

        selected_board = None
        successful_select = False

        while not successful_select:
            try:
                user_input = input(f'\nSelect a board (1-{len(b)}):\n\n{b_str}\n\nBoard number[Default 1]: ')
                user_input = user_input if user_input != '' else 1
                selected_board = b[int(user_input) - 1]
            except (ValueError, KeyError, IndexError):
                print('\n-------------------------')
                print('Invalid number specified.')
                print('-------------------------\n')
            else:
                successful_select = True

        return selected_board



from dataclasses import dataclass
from typing import List, Dict, Iterator

from b_jira_filter.objects.assignee import Assignee
from b_jira_filter.objects.sprint_issue import SprintIssue


@dataclass
class SprintAssignees:
    """
    Container class that holds sprint issues and works with issues assignees.
    """
    completed_sprint_issues: List[SprintIssue]
    not_completed_sprint_issues: List[SprintIssue]

    def aggregate(self) -> Dict[str, Assignee]:
        """
        Aggregates issues by assignees.

        :return: Aggregated issues.
        """
        agg = {}

        for issue in self.completed_sprint_issues + self.not_completed_sprint_issues:
            agg[issue.assignee] = Assignee()

        for issue in self.completed_sprint_issues:
            agg[issue.assignee].add_completed_issue(issue)

        for issue in self.not_completed_sprint_issues:
            agg[issue.assignee].add_not_completed_issue(issue)

        return agg

    def report(self) -> str:
        """
        Creates a human readable report.

        :return: Report string.
        """
        return '\n'.join(list(self.__individual_reports()))

    def __individual_reports(self) -> Iterator[str]:
        """
        Creates and individual report for every assignee.

        :return: Iterator instance over every individual report.
        """
        for key, value in self.aggregate().items():
            name = key.split('#')[1]
            yield f'{name:30s}: {str(value)}'



from b_jira_filter.getters.boards import Boards
from b_jira_filter.getters.sprint_report import SprintReport
from b_jira_filter.getters.sprints import Sprints
from b_jira_filter.objects.credentials import Credentials
from b_jira_filter.objects.sprint_assignees import SprintAssignees


def main() -> None:
    """
    Command that creates a story points report for a specific sprint.

    :return: No return.
    """
    credentials = Credentials()

    board = Boards(credentials.jira_sdk).prompt_select_board()
    sprint = Sprints(credentials.jira_sdk, board).prompt_select_sprint()

    completed, not_completed = SprintReport(credentials, board.id, sprint.id).load()
    sprint_assignees = SprintAssignees(completed, not_completed)

    print('\n\n----------------------------------------------------------------\n\n')
    print(sprint_assignees.report())
    print('\n\n----------------------------------------------------------------\n\n')



from os import listdir
from os.path import isfile, join, isdir, basename
from typing import List, Callable


def common_main(path: str) -> None:
    """
    Command that lists all available commands in given path.

    :param path: Path to show all available commands (files).

    :return: No return.
    """
    items = [join(path, name) for name in listdir(path)]

    """
    Check what are the available commands.
    """

    checks = [
        lambda x: x.endswith('.py')
    ]

    commands = __available_actions(checks, [basename(item) for item in items if isfile(item)])
    if commands:
        print('\nAvailable commands:')
        print(commands)

    """
    Check what are the available sections.
    """

    sections = __available_actions([], [basename(item) for item in items if isdir(item)])
    if sections:
        print('\nAvailable sections:')
        print(sections)


def __available_actions(checks: List[Callable[[str], bool]], items: List[str]) -> str:
    """
    Lists available actions (commands or sections).

    :param checks: A list of functions that check whether the file is an action.
    :param items: Available files as actions.

    :return: Human readable text of available actions.
    """
    checks.append(lambda x: not x.startswith('__'))
    checks.append(lambda x: x != 'main.py')

    commands = []
    for item in items:
        if all([check(item) for check in checks]):
            item = item.replace('.py', '')
            item = f'- {item}'

            commands.append(item)

    return '\n'.join(commands)



import os
from typing import Optional

from jira import JIRA
from jira.resources import GreenHopperResource
from requests.auth import HTTPBasicAuth


class Credentials:
    """
    Credentials class for authentication.
    """
    def __init__(
            self,
            username: Optional[str] = None,
            password: Optional[str] = None,
            server: Optional[str] = None
    ) -> None:
        """
        Constructor.

        :param username: Username for authentication.
        :param password: Password for authentication.
        :param server: Server where your JIRA resources are.
        """
        self.username = username or os.environ.get('JIRA_USERNAME') or input('Username: ')
        self.password = password or os.environ.get('JIRA_PASSWORD') or input('Password: ')
        self.server = server or os.environ.get('JIRA_SERVER') or input('Server: ')

    @property
    def basic_auth(self) -> HTTPBasicAuth:
        """
        Creates basic authorization instance.

        :return: Basic auth instance.
        """
        return HTTPBasicAuth(self.username, self.password)

    @property
    def jira_sdk(self) -> JIRA:
        """
        Creates JIRA SDK instance.

        :return: JIRA SDK instance.
        """
        return JIRA(
            server=self.server,
            basic_auth=(self.username, self.password),
            options={
                'agile_rest_path': GreenHopperResource.AGILE_BASE_REST_PATH
            }
        )



from typing import Dict, Any

from jira import JIRA


class StoryPointsField:
    """
    Class that works with JIRA fields and exposes story points field.
    """
    def __init__(self, jira: JIRA):
        self.__jira = jira

    def get_field(self) -> Dict[str, Any]:
        """
        Gets a field which is considered to be a "story points" field.

        :return: Story points field data.
        """
        return [field for field in self.__jira.fields() if field['name'] == 'Story Points'][0]

    def get_field_id(self):
        """
        Gets story points field id.

        :return: Story points field id.
        """
        return self.get_field()['id']



from typing import List

from jira import JIRA
from jira.resources import Sprint, Board


class Sprints:
    """
    Class that works with JIRA sprints.
    """
    def __init__(self, jira: JIRA, board: Board):
        """
        Constructor.

        :param jira: JIRA SDK instance.
        :param board: A JIRA board in which sprints should be queried.
        """
        self.__jira = jira
        self.__board = board

    def get_sprints(self) -> List[Sprint]:
        """
        Lists all available sprints.

        :return: List of sprints.
        """
        return list(self.__jira.sprints(self.__board.id))

    def format(self, sprints: List[Sprint]) -> str:
        """
        Formats list of sprints into a human readable text.

        :param boards: List of sprints.

        :return: Human readable list.
        """
        return '\n'.join([f'{index + 1}. {str(sprint)}' for index, sprint in enumerate(sprints)])

    def prompt_select_sprint(self) -> Sprint:
        """
        Asks user for an input - asks to select a sprint from available sprints.

        :return: Selected sprint.
        """
        s = self.get_sprints()
        s_str = self.format(s)

        selected_sprint = None
        successful_select = False

        while not successful_select:
            try:
                user_input = input(f'\nSelect a sprint (1-{len(s)}):\n\n{s_str}\n\nSprint number[Default {len(s)}]: ')
                user_input = user_input if user_input != '' else len(s)
                selected_sprint = s[int(user_input) - 1]
            except (ValueError, KeyError, IndexError):
                print('\n-------------------------')
                print('Invalid number specified.')
                print('-------------------------\n')
            else:
                successful_select = True

        return selected_sprint



from setuptools import setup, find_packages

with open('README.md') as file:
    README = file.read()

with open('HISTORY.md') as history_file:
    HISTORY = history_file.read()

with open('VERSION') as file:
    VERSION = file.read()
    VERSION = ''.join(VERSION.split())

setup(
    name='b_jira_filter',
    version=VERSION,
    license='Apache License 2.0',
    packages=find_packages(exclude=[
        # Exclude virtual environment.
        'venv',
        # Exclude test source files.
        'b_jira_filter_test'
    ]),
    entry_points={
        'console_scripts': [
            'jira=b_jira_filter.commands.jira.main:main',
            'jira.sprints=b_jira_filter.commands.jira.sprints.main:main',
            'jira.sprints.storypoints=b_jira_filter.commands.jira.sprints.storypoints:main',
        ],
    },
    description='Script library to interact with JIRA API.',
    long_description=README,
    long_description_content_type="text/markdown",
    include_package_data=True,
    install_requires=[
        'jira>=2.0.0,<3.0.0',
    ],
    author='Laimonas Sutkus',
    author_email='laimonas.sutkus@biomapas.com',
    keywords='JIRA',
    classifiers=[
        'Programming Language :: Python :: 3.8',
        'Operating System :: OS Independent',
    ],
)
